'''OT1 -Single channel- Making up of main reagent stock solution (2 maximum) using a defined solvent'''

from opentrons import robot, containers, instruments
robot.head_speed(x=21000, y=21000, z=5000, a=700, b=700)

class Vector(object):
    def tolist(self):
        return list(self.input_list)

    def astype(self, input_type):
        if input_type == int:
            return Vector([int(float(x)) for x in self.input_list])
        return Vector([input_type(x) for x in self.input_list])

    def __init__(self, input_list):
        self.input_list = input_list

class DataFrame(object):
    def __len__(self):
        return self.length

    def __getitem__(self, value):
        return Vector(self.dict_input[value])

    def __init__(self, dict_input, length):
        self.dict_input = dict_input
        self.length = length

# Function that reads a csv file correctly without having to import anything (issues with molport). Uses 2 classes, Vector and DataFrame
def read_csv(input_file):
    lines = open(input_file).readlines()
    header = lines[0].rstrip().split(",")
    out_d = {}
    for head in header:
        out_d[head] = []
    for line in lines[1:]:
        spl_line = line.rstrip().split(",")
        for i, head in enumerate(header):
            out_d[head].append(spl_line[i])
    df = DataFrame(out_d, len(lines[1:]))
    return df

# CSV file data
reaction_conditions_df = read_csv(
    r"C:\Users\opentrons\protocols\GitHub_repos\OT1-coding\PHIP\Iteration_3-5\csv\reaction_conditions_final.csv")
#reactant_df = read_csv(
#    r"C:\Users\opentrons\protocols\GitHub_repos\OT1-coding\PHIP\Iteration_3-5\csv\3-5_imidazoliums.csv")
reactant_df = read_csv(
    r"C:\Users\opentrons\protocols\GitHub_repos\OT1-coding\PHIP\Iteration_3-5\csv\3-5_acids.csv")
#reactant_df = read_csv(
#     r"C:\Users\opentrons\protocols\GitHub_repos\OT1-coding\PHIP\Iteration_3-5\csv\3-5_amines.csv")

"""Function that does one liquid handling transfers. Dilute a solid reagent in trough or a rack to the right concentration.
It requires 2 csv files, reaction condition and the list of reagents."""

def stock_solution_main(reaction_conditions, reactant):
    # Deck setup
    tiprack_1000 = containers.load("tiprack-1000ul-H", "D3")
    tiprack_1000_2 = containers.load("tiprack-1000ul-H", "E1")

    solvent_source_trough4row = containers.load("trough-12row", "C2")
    
    destination_trough4row = containers.load("trough-12row", "C1")
    destination_rack_stock_reactants_1 = containers.load("FluidX_24_5ml", "A1", "R_1")
    destination_rack_stock_reactants_2 = containers.load("FluidX_24_5ml", "A2", "R_2")
    destination_rack_stock_reactants_3 = containers.load("FluidX_24_5ml", "B1", "R_3")
    destination_rack_stock_reactants_4 = containers.load("FluidX_24_5ml", "B2", "R_4")
    trash = containers.load("point", "B3")
    # Pipettes SetUp
    p1000 = instruments.Pipette(
        name='eppendorf1000',
        axis='b',
        trash_container=trash,
        tip_racks=[tiprack_1000,tiprack_1000_2],
        max_volume=1000,
        min_volume=30,
        channels=1,
    )
    # from reaction_conditions_df
    reaction_header = "reaction"
    reaction_to_start = "Urea-Dboc-Coupling_sequence"
    solvent_trough_location_header_reactant1 = "Reactant 1 - Stock solution solvent location"
    solvent_trough_location_header_reactant2 = "Reactant 2 - Stock solution solvent location"
    solvent_trough_location_header_reactant3 = "Reactant 3 - Stock solution solvent location"
    
    # from reactant_df
    rack_ID_header = "Rack ID"
    id_header = "CPD ID"
    reactant_type_header = "Reactant type"
    reactant_type_1 = "reactant 1"
    reactant_type_2 = "reactant 2"
    reactant_type_3 = "reactant 3"
    trough_12 = "trough_12"
    trough_4 = "trough_4"
    rack_1 = "24_rack1"
    rack_2 = "24_rack2"
    rack_3 = "24_rack3"
    rack_4 = "24_rack4"
    destination_location_header = "Location"
    volume_to_add_stock_header = "Initial volume to add -exp (uL)"
    
    for index, value in enumerate(reaction_conditions_df[reaction_header].tolist()):
        if value == reaction_to_start:
            if reactant_df[reactant_type_header].tolist()[0] == "reactant 1":
                solvent_location = reaction_conditions_df[solvent_trough_location_header_reactant1].tolist()[index]
            elif reactant_df[reactant_type_header].tolist()[0] == "reactant 2":
                solvent_location = reaction_conditions_df[solvent_trough_location_header_reactant2].tolist()[index]
            elif reactant_df[reactant_type_header].tolist()[0] == "reactant 3":
                solvent_location = reaction_conditions_df[solvent_trough_location_header_reactant2].tolist()[index]
            else:
                print ("error")
        else:
            continue
            
    p1000.pick_up_tip()
    for i, x in enumerate(reactant_df[destination_location_header].tolist()):
        destination_location = x
        vol_to_dispense = [reactant_df[volume_to_add_stock_header].tolist()[i]]
        reactants_id = reactant_df[rack_ID_header].tolist()[i]
        if reactants_id == "":
            break
        if reactants_id == trough_12:
            if vol_to_dispense != 0:
                p1000.transfer(vol_to_dispense, solvent_source_trough4row.wells(solvent_location),
                               destination_trough4row.wells(destination_location).top(-5), new_tip='never',air_gap = 10)
        
        if reactants_id == trough_4:
            if vol_to_dispense != 0:
                p1000.transfer(vol_to_dispense, solvent_source_trough4row.wells(solvent_location),
                               destination_trough4row.wells(destination_location).top(-5), new_tip='never',air_gap = 10)

        if reactants_id == rack_1:
            if vol_to_dispense != 0:
                p1000.transfer(vol_to_dispense, solvent_source_trough4row.wells(solvent_location),
                               destination_rack_stock_reactants_1.wells(destination_location).top(-5), new_tip='never',air_gap = 10)
        
        if reactants_id == rack_2:
            if vol_to_dispense != 0:
                p1000.transfer(vol_to_dispense, solvent_source_trough4row.wells(solvent_location),
                               destination_rack_stock_reactants_2.wells(destination_location).top(-5), new_tip='never',air_gap = 10)
        
        if reactants_id == rack_3:
            if vol_to_dispense != 0:
                p1000.transfer(vol_to_dispense, solvent_source_trough4row.wells(solvent_location),
                               destination_rack_stock_reactants_3.wells(destination_location).top(-5), new_tip='never',air_gap = 10,)
        
        if reactants_id == rack_4:
            if vol_to_dispense != 0:
                p1000.transfer(vol_to_dispense, solvent_source_trough4row.wells(solvent_location),
                               destination_rack_stock_reactants_4.wells(destination_location).top(-5), new_tip='never',air_gap = 10,)
    p1000.drop_tip()
    robot.home()    
stock_solution_main(reaction_conditions_df, reactant_df)
