"""Protocol that used the multichannel to add base and coupling agent to the desired number of rows in the plate."""

from opentrons import robot, containers, instruments
robot.head_speed(x=21000, y=21000, z=5000, a=700, b=700)

class Vector(object):
    def tolist(self):
        return list(self.input_list)

    def astype(self, input_type):
        if input_type == int:
            return Vector([int(float(x)) for x in self.input_list])
        return Vector([input_type(x) for x in self.input_list])

    def __init__(self, input_list):
        self.input_list = input_list

class DataFrame(object):
    def __len__(self):
        return self.length

    def __getitem__(self, value):
        return Vector(self.dict_input[value])

    def __init__(self, dict_input, length):
        self.dict_input = dict_input
        self.length = length
#Function that reads a csv file correctly without having to import anything (issues with molport). Uses 2 classes, Vector and DataFrame
def read_csv(input_file):
    lines = open(input_file).readlines()
    header = lines[0].rstrip().split(",")
    out_d = {}
    for head in header:
        out_d[head] = []
    for line in lines[1:]:
        spl_line = line.rstrip().split(",")
        for i, head in enumerate(header):
            out_d[head].append(spl_line[i])
    df = DataFrame(out_d, len(lines[1:]))
    return df
#CSV file data
reaction_conditions_df = read_csv(
    r"C:\Users\opentrons\protocols\GitHub_repos\OT1-coding\PHIP\Iteration_3-5\csv\reaction_conditions_final.csv")
reactant_df = read_csv(
    r"C:\Users\opentrons\protocols\GitHub_repos\OT1-coding\PHIP\Iteration_3-5\csv\3-5_imidazoliums.csv")


# Deck setup
tiprack_300 = containers.load("tiprack-300ul", "D3")
source_trough12row = containers.load('trough-12row', "E2")
reaction_rack = containers.load("Starlab_96_Square_2mL", "D1")

trash = containers.load("point", "B3")

# Pipettes SetUp
p300_multi = instruments.Pipette(
    name='dlab_300multi',
    axis="a",
    trash_container=trash,
    tip_racks=[tiprack_300],
    max_volume=300,
    min_volume=30,
    channels=8,
)

def add_reagent(reaction_condition, reactant):

    id_header = "reaction"
    source_location_header = "Location"
    reaction_to_start = "Urea-Dboc-Coupling_sequence"
    rows_number_header = "Reactant 2 - row addition"
    vol_to_add_reactant_1_header = "Reactant 1 - volume to add per well (uL)"
    vol_to_add_reactant_2_header = "Reactant 2 - volume to add per well (uL)"
    vol_to_add_reactant_3_header = "Reactant 3 - volume to add per well (uL)"
    number_rows = 7
    source_location = reactant_df[source_location_header].tolist()[2]
    
    for index, value in enumerate(reaction_conditions_df[id_header].tolist()):
        if value == reaction_to_start:
            volume_to_add = float(reaction_conditions_df[vol_to_add_reactant_2_header].tolist()[index])
    
    p300_multi.distribute(volume_to_add, source_trough12row.wells(source_location),
                          [x.top() for x in reaction_rack.rows(0, to=number_rows)], air_gap = 10)
    
    robot.home()
add_reagent(reaction_conditions_df,reactant_df)
