"""Protocol that used the multichannel to add base and coupling agent to the desired number of rows in the plate."""

from opentrons import robot, containers, instruments
#robot.reset()
robot.head_speed(x=16000, y=16000, z=3000, a=700, b=700)

class Vector(object):
    def tolist(self):
        return list(self.input_list)

    def astype(self, input_type):
        if input_type == int:
            return Vector([int(float(x)) for x in self.input_list])
        return Vector([input_type(x) for x in self.input_list])

    def __init__(self, input_list):
        self.input_list = input_list

class DataFrame(object):
    def __len__(self):
        return self.length

    def __getitem__(self, value):
        return Vector(self.dict_input[value])

    def __init__(self, dict_input, length):
        self.dict_input = dict_input
        self.length = length
#Function that reads a csv file correctly without having to import anything (issues with molport). Uses 2 classes, Vector and DataFrame
def read_csv(input_file):
    lines = open(input_file).readlines()
    header = lines[0].rstrip().split(",")
    out_d = {}
    for head in header:
        out_d[head] = []
    for line in lines[1:]:
        spl_line = line.rstrip().split(",")
        for i, head in enumerate(header):
            out_d[head].append(spl_line[i])
    df = DataFrame(out_d, len(lines[1:]))
    return df
#CSV file data
reaction_conditions_df = read_csv(
    r"C:\Users\opentrons\protocols\GitHub_repos\OT1-coding\PHIP\Iteration_3-5\csv\reaction_conditions_final.csv")

# Deck setup
tiprack_300 = containers.load("tiprack-300ul", "D3")
source_trough12row = containers.load('trough-12row', "E2")
reaction_rack = containers.load("Starlab_96_Square_2mL", "D1")
trash = containers.load("point", "B3")

# Pipettes SetUp
p300_multi = instruments.Pipette(
    name='dlab_300multi',
    axis="a",
    trash_container=trash,
    tip_racks=[tiprack_300],
    max_volume=300,
    min_volume=30,
    channels=8,
)

def add_base_reagent(reaction_condition):

    id_header = "reaction"
    reaction_to_start = "Urea-Dboc-Coupling_sequence"
    reagent_trough_location_header = "step 2 split solvent location"
    volume_reagent_header = "step 2 split solvent volume"
    number_reaction_per_plate_header = "Step 1 number wells per plate"
    
    for index, value in enumerate(reaction_conditions_df[id_header].tolist()):
        if value == reaction_to_start:
            number_reaction_per_plate = int(reaction_conditions_df[number_reaction_per_plate_header].tolist()[index])
            volume_reagent = float(reaction_conditions_df[volume_reagent_header].tolist()[index])
            reagent_trough_location = reaction_conditions_df[reagent_trough_location_header].tolist()[index]
        else:
            print ("not the right reaction sequence")


    number_rows_to_dispense = int(-(-number_reaction_per_plate // 8))
    #print (number_reaction_per_plate, type(number_reaction_per_plate), "-", number_rows_to_dispense)
    p300_multi.distribute(volume_reagent, source_trough12row.wells(reagent_trough_location),
                          [x.top() for x in reaction_rack.rows(0, to=number_rows_to_dispense-1)], air_gap =10)

    for i in range(0, number_rows_to_dispense):
        p300_multi.pick_up_tip()
        p300_multi.mix(10, 300, reaction_rack.rows(i).bottom(1.5))
        p300_multi.blow_out(reaction_rack.rows(i).top(-10))
        p300_multi.drop_tip()
    robot.home()
    
add_base_reagent(reaction_conditions_df)
#robot.commands()