"""Protocol that used the multichannel to add base and coupling agent to the desired number of rows in the plate."""

from opentrons import robot, containers, instruments
robot.head_speed(x=24000, y=24000, z=4000, a=700, b=700)

class Vector(object):
    def tolist(self):
        return list(self.input_list)

    def astype(self, input_type):
        if input_type == int:
            return Vector([int(float(x)) for x in self.input_list])
        return Vector([input_type(x) for x in self.input_list])

    def __init__(self, input_list):
        self.input_list = input_list

class DataFrame(object):
    def __len__(self):
        return self.length

    def __getitem__(self, value):
        return Vector(self.dict_input[value])

    def __init__(self, dict_input, length):
        self.dict_input = dict_input
        self.length = length
#Function that reads a csv file correctly without having to import anything (issues with molport). Uses 2 classes, Vector and DataFrame
def read_csv(input_file):
    lines = open(input_file).readlines()
    header = lines[0].rstrip().split(",")
    out_d = {}
    for head in header:
        out_d[head] = []
    for line in lines[1:]:
        spl_line = line.rstrip().split(",")
        for i, head in enumerate(header):
            out_d[head].append(spl_line[i])
    df = DataFrame(out_d, len(lines[1:]))
    return df
#CSV file data
reaction_conditions_df = read_csv(
    r"C:\Users\opentrons\protocols\GitHub_repos\OT1-coding\PHIP\Iteration_3-5\csv\reaction_conditions_final.csv")

# Deck setup
tiprack_300 = containers.load("tiprack-300ul", "D3")
tiprack_300_2 = containers.load("tiprack-300ul", "E3")
tiprack_300_3 = containers.load("tiprack-300ul", "B2")


source_trough12row = containers.load('trough-12row', "E2")
destination_reaction_rack_1 = containers.load("Starlab_96_Square_2mL", "B1")
destination_reaction_rack_2 = containers.load("Starlab_96_Square_2mL", "C1")
destination_reaction_rack_3 = containers.load("Starlab_96_Square_2mL", "D1")

trash = containers.load("point", "B3")

# Pipettes SetUp

p300_multi  = instruments.Pipette(
    name='dlab_300multi',
    axis="a",
    trash_container=trash,
    tip_racks=[tiprack_300, tiprack_300_2],
    max_volume=300,
    min_volume=30,
    channels=8,
)

def add_base_reagent(reaction_condition):

    id_header = "reaction"
    reaction_to_start = "Urea-Dboc-Coupling_sequence"
    volume_solvent_header = "Step 3 solvent volume"
    volume_reagent_1_header = "Step 3 reagent 1 -volume to dispense"
    volume_reagent_2_header = "Step 3 reagent 2 -volume to dispense"
    location_solvent_header = "Step 3 solvent location"
    location_reagent_1_header = "Step 3 reagent 1 - location"
    location_reagent_2_header = "Step 3 reagent 2 - location"
    number_rows_plate_1_header = "Step 3 - Plate 1 number row"
    number_rows_plate_2_header = "Step 3 - Plate 2 number row"
    number_rows_plate_3_header = "Step 3 - Plate 3 number row"

    
    for index, value in enumerate(reaction_conditions_df[id_header].tolist()):
        if value == reaction_to_start:
            
            number_rows_plate_1 = int(reaction_conditions_df[number_rows_plate_1_header].tolist()[index])
            number_rows_plate_2 = int(reaction_conditions_df[number_rows_plate_2_header].tolist()[index])
            number_rows_plate_3 = int(reaction_conditions_df[number_rows_plate_3_header].tolist()[index])
            location_reagent_1 = reaction_conditions_df[location_reagent_1_header].tolist()[index]
            location_reagent_2 = reaction_conditions_df[location_reagent_2_header].tolist()[index]
            location_solvent = reaction_conditions_df[location_solvent_header].tolist()[index]
            volume_to_add_solvent = float(reaction_conditions_df[volume_solvent_header].tolist()[index])
            volume_to_add_reagent_1 = float(reaction_conditions_df[volume_reagent_1_header].tolist()[index])
            volume_to_add_reagent_2 = float(reaction_conditions_df[volume_reagent_2_header].tolist()[index])
        else:
            print ("not the right reaction sequence")

    p300_multi.distribute(volume_to_add_solvent, source_trough12row.wells(location_solvent), [x.top(-10) for x in destination_reaction_rack_1.rows(0, to=number_rows_plate_1-1)], air_gap = 15)
    p300_multi.distribute(volume_to_add_solvent, source_trough12row.wells(location_solvent), [x.top(-10) for x in destination_reaction_rack_2.rows(0, to=number_rows_plate_2-1)], air_gap = 15)
    p300_multi.distribute(volume_to_add_solvent, source_trough12row.wells(location_solvent), [x.top(-10) for x in destination_reaction_rack_3.rows(0, to=number_rows_plate_3-1)], air_gap = 15)
    
    robot.home()
    
add_base_reagent(reaction_conditions_df)
