from opentrons import robot, containers, instruments
robot.head_speed(x=20000, y=20000, z=4000, a=400, b=400)

class Vector(object):
    def tolist(self):
        return list(self.input_list)

    def astype(self, input_type):
        if input_type == int:
            return Vector([int(float(x)) for x in self.input_list])
        return Vector([input_type(x) for x in self.input_list])

    def __init__(self, input_list):
        self.input_list = input_list

class DataFrame(object):
    def __len__(self):
        return self.length

    def __getitem__(self, value):
        return Vector(self.dict_input[value])

    def __init__(self, dict_input, length):
        self.dict_input = dict_input
        self.length = length

# Function that reads a csv file correctly without having to import anything (issues with molport). Uses 2 classes, Vector and DataFrame
def read_csv(input_file):
    lines = open(input_file).readlines()
    header = lines[0].rstrip().split(",")
    out_d = {}
    for head in header:
        out_d[head] = []
    for line in lines[1:]:
        spl_line = line.rstrip().split(",")
        for i, head in enumerate(header):
            out_d[head].append(spl_line[i])
    df = DataFrame(out_d, len(lines[1:]))
    return df


# CSV file data
reaction_conditions_df = read_csv(
    r"C:\Users\opentrons\protocols\GitHub_repos\OT1-coding\PHIP\Iteration_4.2\csv\reaction_conditions.csv")

# Deck setup
tiprack_1000 = containers.load("tiprack-1000ul-H", "D3")
rack_stock_reactants = containers.load("Starlab_96_Square_2mL", "A1")

destination_reaction_rack_1 = containers.load("Starlab_96_Square_2mL", "B1")
destination_reaction_rack_2 = containers.load("Starlab_96_Square_2mL", "C1")
destination_reaction_rack_3 = containers.load("Starlab_96_Square_2mL", "D1")

trash = containers.load("point", "B3")

# Pipettes SetUp
p1000 = instruments.Pipette(
    name='eppendorf1000',
    axis='b',
    trash_container=trash,
    tip_racks=[tiprack_1000],
    max_volume=1000,
    min_volume=30,
    channels=1,
)

id_header = "reaction"
reaction_to_start = "Urea-Dboc/Sulfo-esterolysis/Coupling_sequence"
volume_reactant_2_header = "3. Reactant 2 - volume to add per well (uL)"
number_rows_plate_1_header = "3. Plate 1 number of rows"
number_rows_plate_2_header = "3. Plate 2 number of rows"
number_rows_plate_3_header = "3. Plate 3 number of rows"
rack_ID_header = "Rack ID"
#slight shortcut. The step count shoul dbe taken from the csv file.
step = 24

for index, value in enumerate(reaction_conditions_df[id_header].tolist()):
    if value == reaction_to_start:
        number_rows_plate_1 = int(reaction_conditions_df[number_rows_plate_1_header].tolist()[index])
        number_rows_plate_2 = int(reaction_conditions_df[number_rows_plate_2_header].tolist()[index])
        volume_to_add_reactant_2 = float(reaction_conditions_df[volume_reactant_2_header].tolist()[index])
    else:
        print ("not the right reaction sequence")

for index in range(0, 20):
    location_reactant = index
    destination_list = [destination_reaction_rack_1.wells(index), destination_reaction_rack_1.wells(index + step),
                        destination_reaction_rack_1.wells(index + step * 2), destination_reaction_rack_1.wells(index + step * 3), destination_reaction_rack_2.wells(index),
                        destination_reaction_rack_2.wells(index + step),
                        destination_reaction_rack_2.wells(index + step * 2),destination_reaction_rack_2.wells(index + step * 3)]
    p1000.distribute(volume_to_add_reactant_2, rack_stock_reactants.wells(location_reactant).bottom(2),
                     destination_list, air_gap=30)
