from opentrons import robot, containers, instruments
robot.head_speed(x=18000, y=18000, z=5000, a=700, b=700)


class Vector(object):
    def tolist(self):
        return list(self.input_list)

    def astype(self, input_type):
        if input_type == int:
            return Vector([int(float(x)) for x in self.input_list])
        return Vector([input_type(x) for x in self.input_list])

    def __init__(self, input_list):
        self.input_list = input_list


class DataFrame(object):
    def __len__(self):
        return self.length

    def __getitem__(self, value):
        return Vector(self.dict_input[value])

    def __init__(self, dict_input, length):
        self.dict_input = dict_input
        self.length = length


# Function that reads a csv file correctly without having to import anything (issues with molport). Uses 2 classes, Vector and DataFrame
def read_csv(input_file):
    lines = open(input_file).readlines()
    header = lines[0].rstrip().split(",")
    out_d = {}
    for head in header:
        out_d[head] = []
    for line in lines[1:]:
        spl_line = line.rstrip().split(",")
        for i, head in enumerate(header):
            out_d[head].append(spl_line[i])
    df = DataFrame(out_d, len(lines[1:]))
    return df

# CSV file data
reactant_df = read_csv(r"C:\Users\opentrons\protocols\GitHub_repos\OT1-coding\PHIP\Iteration_4.2\csv\4-2_amines_sulfonamides.csv")
reaction_conditions_df = read_csv(r"C:\Users\opentrons\protocols\GitHub_repos\OT1-coding\PHIP\Iteration_4.2\csv\reaction_conditions.csv")

def reactants_transfer(reactants, solvent):
    # Deck setup
    tiprack_1000 = containers.load("tiprack-1000ul-H", "D3")
    source_trough12row = containers.load('trough-12row', "E2")
    rack_stock_reactants_1 = containers.load("FluidX_24", "A1", "R_1")
    rack_stock_reactants_2 = containers.load("FluidX_24", "A2", "R_2")
    rack_stock_reactants_3 = containers.load("FluidX_24", "B1", "R_3")
    rack_stock_reactants_4 = containers.load("FluidX_24", "B2", "R_4")
    reaction_rack = containers.load("Starlab_96_Square_2mL", "D1")

    trash = containers.load("point", "B3")

    # Pipettes SetUp
    p1000 = instruments.Pipette(
        name='eppendorf1000',
        axis='b',
        trash_container=trash,
        tip_racks=[tiprack_1000],
        max_volume=1000,
        min_volume=30,
        channels=1,
    )
    
    
    id_header = "reaction"
    rack_ID_header = "Rack ID"
    source_location_header = "Location"
    reaction_to_start = "Urea-Dboc/Sulfo-esterolysis/Coupling_sequence"
    rows_number_header = "Reactant 3 - row addition"
    vol_to_add_reactant_1_header = "Reactant 1 - volume to add per well (uL)"
    vol_to_add_reactant_2_header = "Reactant 2 - volume to add per well (uL)"
    vol_to_add_reactant_3_header = "Reactant 3 - volume to add per well (uL)"
    vol_to_add_reactant_4_header = "Reactant 4 - volume to add per well (uL)"

    for i, x in enumerate(reaction_conditions_df[id_header].tolist()):
        if x == reaction_to_start:
            volume_to_add = float(reaction_conditions_df[vol_to_add_reactant_4_header].tolist()[i])
        else:
            print("error")
    
    for i, x in enumerate(reactant_df[rack_ID_header].tolist()):
        source_location = reactant_df[source_location_header].tolist()[i]
        if x == "":
            break
        if x == "24_rack1":
            p1000.transfer(volume_to_add, rack_stock_reactants_1.wells(source_location),
                           reaction_rack.wells(i).top(-5), air_gap=50)
        if x == "24_rack2":
            p1000.transfer(volume_to_add, rack_stock_reactants_2.wells(source_location),
                           reaction_rack.wells(i).top(-5), air_gap=50)
        if x == "24_rack3":
            p1000.transfer(volume_to_add, rack_stock_reactants_3.wells(source_location),
                           reaction_rack.wells(i).top(-5), air_gap=50)
        if x == "24_rack4":
            p1000.transfer(volume_to_add, rack_stock_reactants_4.wells(source_location),
                           reaction_rack.wells(i).top(-5), air_gap=50)

reactants_transfer(reactant_df, reaction_conditions_df)
