"""Protocol that used the multichannel to add base and coupling agent to the desired number of rows in the plate."""

from opentrons import robot, containers, instruments
robot.head_speed(x=16000, y=16000, z=3000, a=400, b=400)

class Vector(object):
    def tolist(self):
        return list(self.input_list)

    def astype(self, input_type):
        if input_type == int:
            return Vector([int(float(x)) for x in self.input_list])
        return Vector([input_type(x) for x in self.input_list])

    def __init__(self, input_list):
        self.input_list = input_list

class DataFrame(object):
    def __len__(self):
        return self.length

    def __getitem__(self, value):
        return Vector(self.dict_input[value])

    def __init__(self, dict_input, length):
        self.dict_input = dict_input
        self.length = length
#Function that reads a csv file correctly without having to import anything (issues with molport). Uses 2 classes, Vector and DataFrame
def read_csv(input_file):
    lines = open(input_file).readlines()
    header = lines[0].rstrip().split(",")
    out_d = {}
    for head in header:
        out_d[head] = []
    for line in lines[1:]:
        spl_line = line.rstrip().split(",")
        for i, head in enumerate(header):
            out_d[head].append(spl_line[i])
    df = DataFrame(out_d, len(lines[1:]))
    return df

#CSV file data
reaction_conditions_df = read_csv(r"C:\Users\opentrons\protocols\GitHub_repos\OT1-coding\PHIP\Iteration_4.2\csv\reaction_conditions.csv")

# Deck setup
tiprack_300 = containers.load("tiprack-300ul", "D3")
source_trough12row = containers.load('trough-12row', "E2")
reaction_rack = containers.load("Starlab_96_Square_2mL", "D1")
trash = containers.load("point", "B3")

# Pipettes SetUp
p300_multi = instruments.Pipette(
    name='dlab_300multi',
    axis="a",
    trash_container=trash,
    tip_racks=[tiprack_300],
    max_volume=300,
    min_volume=30,
    channels=8,
)

def add_base_coupling(reaction_condition):
    
    id_header = "reaction"
    reaction_to_start = "Urea-Dboc/Sulfo-esterolysis/Coupling_sequence"
    solvent_volume_header = "hydrolysis solvent to add - volume"
    location_solvent_header = "hydrolysis solvent to add - location"
    reagent_1_location_header = "hydrolysis - Reagent 1 - location"
    reagent_1_volume_header ="hydrolysis - Reagent 1 - volume"
    number_rows = 3

    for i, x in enumerate(reaction_conditions_df[id_header].tolist()):
        if x == reaction_to_start:
            solvent_location = reaction_conditions_df[location_solvent_header].tolist()[i]
            solvent_volume_to_add = float(reaction_conditions_df[solvent_volume_header].tolist()[i])
            reagent_1_location = reaction_conditions_df[reagent_1_location_header].tolist()[i]
            reagent_1_volume_to_add = float(reaction_conditions_df[reagent_1_volume_header].tolist()[i])
        else:
            print("not the right reaction sequence")
    p300_multi.distribute(solvent_volume_to_add, source_trough12row.wells(solvent_location),
                                  [x.top() for x in reaction_rack.rows(0, to=number_rows-1)], air_gap=10)
    p300_multi.distribute(reagent_1_volume_to_add, source_trough12row.wells(reagent_1_location),
                          [x.top() for x in reaction_rack.rows(0, to=number_rows - 1)], air_gap=10)
    robot.home()
add_base_coupling(reaction_conditions_df)
