from opentrons import robot, containers, instruments
robot.head_speed(x=21000,  y=21000,  z=4000, a=700, b=700)
class Vector(object):
    def tolist(self):
        return list(self.input_list)

    def astype(self, input_type):
        if input_type == int:
            return Vector([int(float(x)) for x in self.input_list])
        return Vector([input_type(x) for x in self.input_list])

    def __init__(self, input_list):
        self.input_list = input_list

class DataFrame(object):
    def __len__(self):
        return self.length

    def __getitem__(self, value):
        return Vector(self.dict_input[value])

    def __init__(self, dict_input, length):
        self.dict_input = dict_input
        self.length = length
#Function that reads a csv file correctly without having to import anything (issues with molport). Uses 2 classes, Vector and DataFrame
def read_csv(input_file):
    lines = open(input_file).readlines()
    header = lines[0].rstrip().split(",")
    out_d = {}
    for head in header:
        out_d[head] = []
    for line in lines[1:]:
        spl_line = line.rstrip().split(",")
        for i, head in enumerate(header):
            out_d[head].append(spl_line[i])
    df = DataFrame(out_d, len(lines[1:]))
    return df

#Deck setup
tiprack_300 = containers.load("tiprack-300ul", "D3")
tiprack_300_2 = containers.load("tiprack-300ul", "E2")
source_trough4row = containers.load("trough-12row", "C2")
reaction_rack = containers.load("Starlab_96_Square_2mL", "D1")
trash = containers.load("point", "B3")

reactant_df = read_csv(
    r"C:\Users\opentrons\protocols\GitHub_repos\OT1-coding\PHIP\Iteration_4.2\csv\4-2_Amines_ureas.csv")

#Pipettes SetUp
p300_multi  = instruments.Pipette(
    name='dlab_300multi',
    axis="a",
    trash_container=trash,
    tip_racks=[tiprack_300, tiprack_300_2],
    max_volume=300,
    min_volume=30,
    channels=8,
)
workup_type_header = "Workup type"
acid_wup = "acid"
neutral_wup = "neutral"

location_solvent = "A5"
volume_solvent = 300
location_aqueous_basic = "A8"
volume_aqueous_basic = 300
location_aqueous_acid = "A2"
volume_aqueous_acid = 300
number_rows = 8


def work_up_acid_or_neutral(reactant):

    for index, value in enumerate (reactant[workup_type_header].tolist()):
        if value == acid_wup:
            p300_multi.transfer(volume_aqueous_acid, source_trough4row.wells(location_aqueous_acid), reaction_rack.rows(index).top(), air_gap=10)            
        elif value == neutral_wup:
            p300_multi.distribute(volume_aqueous_basic, source_trough4row.wells(location_aqueous_basic), reaction_rack.rows(index).top(), air_gap=10)
            
        else:
            print("error")
    p300_multi.pick_up_tip()
    p300_multi.aspirate (300, source_trough4row.wells(location_solvent))
    p300_multi.dispense (source_trough4row.wells(location_solvent))
    p300_multi.distribute(volume_solvent, source_trough4row.wells(location_solvent), [x.top() for x in reaction_rack.rows(0,to=number_rows-1)], air_gap=10)
    robot.home()

work_up_acid_or_neutral(reactant_df)
