"""Protocol that used the multichannel to add base and coupling agent to the desired number of rows in the plate."""

from opentrons import robot, containers, instruments
robot.head_speed(x=20000, y=20000, z=2000, a=400, b=300)

class Vector(object):
    def tolist(self):
        return list(self.input_list)

    def astype(self, input_type):
        if input_type == int:
            return Vector([int(float(x)) for x in self.input_list])
        return Vector([input_type(x) for x in self.input_list])

    def __init__(self, input_list):
        self.input_list = input_list

class DataFrame(object):
    def __len__(self):
        return self.length

    def __getitem__(self, value):
        return Vector(self.dict_input[value])

    def __init__(self, dict_input, length):
        self.dict_input = dict_input
        self.length = length
#Function that reads a csv file correctly without having to import anything (issues with molport). Uses 2 classes, Vector and DataFrame
def read_csv(input_file):
    lines = open(input_file).readlines()
    header = lines[0].rstrip().split(",")
    out_d = {}
    for head in header:
        out_d[head] = []
    for line in lines[1:]:
        spl_line = line.rstrip().split(",")
        for i, head in enumerate(header):
            out_d[head].append(spl_line[i])
    df = DataFrame(out_d, len(lines[1:]))
    return df
#CSV file data
reaction_conditions_df = read_csv(
    r"C:\Users\opentrons\protocols\GitHub_repos\OT1-coding\PHIP\Iteration_4.2\csv\reaction_conditions.csv")

# Deck setup
tiprack_300 = containers.load("tiprack-300ul", "D3")
tiprack_300_2 = containers.load("tiprack-300ul", "E3")
tiprack_300_3 = containers.load("tiprack-300ul", "B2")


source_trough12row = containers.load('trough-12row', "E2")
reaction_rack = containers.load("Starlab_96_Square_2mL", "B1")
destination_reaction_rack_1 = containers.load("Starlab_96_Square_2mL", "E1")
destination_reaction_rack_2 = containers.load("Starlab_96_Square_2mL", "D1")
destination_reaction_rack_3 = containers.load("Starlab_96_Square_2mL", "C1")

trash = containers.load("point", "B3")

# Pipettes SetUp
p300_multi  = instruments.Pipette(
    name='dlab_300multi_no_min',
    axis="a",
    trash_container=trash,
    tip_racks=[tiprack_300,tiprack_300_2,tiprack_300_3],
    max_volume=300,
    min_volume=0,
    channels=8,
)

def add_base_reagent(reaction_condition):

    id_header = "reaction"
    reaction_to_start = "Urea-Dboc/Sulfo-esterolysis/Coupling_sequence"
    volume_reagent_header = "1.2 - volume per transfer"
    number_reaction_per_plate_header = "1.2 - number wells per plate"
    number_destination_per_source_header = "1.2 - Number of wells destination per well source - translates in rows"
    
    for index, value in enumerate(reaction_conditions_df[id_header].tolist()):
        if value == reaction_to_start:
            number_reaction_per_plate = int(reaction_conditions_df[number_reaction_per_plate_header].tolist()[index])
            number_destination = int(reaction_conditions_df[number_destination_per_source_header].tolist()[index])
            volume_to_add = float(reaction_conditions_df[volume_reagent_header].tolist()[index])
        else:
            print ("not the right reaction sequence")
    number_rows_source = int(-(-number_reaction_per_plate // 8))
    n=0
    for i in range (0,4):
        p300_multi.pick_up_tip()
        p300_multi.aspirate(250, reaction_rack.rows(i).bottom(5))
        p300_multi.dispense(250, reaction_rack.rows(i).bottom(5))
        p300_multi.distribute(volume_to_add, reaction_rack.rows(i).bottom(), [x.top(-10) for x in destination_reaction_rack_1.rows(n, to=n+number_destination-1)], air_gap = 30, blow_out = True)
        n=n+number_destination
    n=0
    for i in range (4,8):
        p300_multi.pick_up_tip()
        p300_multi.aspirate(250, reaction_rack.rows(i).bottom(5))
        p300_multi.dispense(250, reaction_rack.rows(i).bottom(5))
        p300_multi.distribute(volume_to_add, reaction_rack.rows(i).bottom(), [x.top(-10) for x in destination_reaction_rack_2.rows(n, to=n+number_destination-1)], air_gap = 30, blow_out = True)
        n=n+number_destination
    robot.home()
    
add_base_reagent(reaction_conditions_df)
